'use strict';

var request = require('request');
var parallel = require('run-parallel');
var chalk = require('chalk');

var baseUri = 'https://api.pushbullet.com/v2';

module.exports = function (config) {
  return function sendPushNotification(_ref, cb) {
    var subject = _ref.subject;
    var message = _ref.message;

    getDevices(function (err, res, body) {
      if (err || res.statusCode !== 200) {
        console.log(chalk.red('Error while retrieving devices for push notification.'), err);
        return cb();
      }
      var notifications = JSON.parse(body).devices.filter(function (d) {
        return d.active;
      }).map(function (el) {
        return function (cb) {
          sendPush({ type: 'note', device_iden: el.iden, title: subject, body: message }, function (err, res, body) {
            if (err || res.statusCode !== 200) {
              console.log(chalk.red('Error while sending push notification.'), err);
            } else {
              console.log(chalk.yellow('Push notification sent to ' + el.nickname + '.'));
            }
            cb();
          });
        };
      });
      parallel(notifications, cb);
    });
  };

  function doRequest(_ref2, cb) {
    var method = _ref2.method;
    var uri = _ref2.uri;
    var data = _ref2.data;

    request({
      method: method,
      uri: baseUri + uri,
      formData: data,
      headers: {
        'Access-Token': config.accessToken
      }
    }, cb);
  }

  function getDevices(cb) {
    doRequest({
      method: 'GET',
      uri: '/devices'
    }, cb);
  }

  function sendPush(data, cb) {
    doRequest({
      method: 'POST',
      uri: '/pushes',
      data: data
    }, cb);
  }
};